/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/06/06
 * Description        : Main program body.
*********************************************************************************
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* Attention: This software (modified or not) and binary are used for 
* microcontroller manufactured by Nanjing Qinheng Microelectronics.
*******************************************************************************/
/*
 *@Note
 SPI DMA, master/slave mode transceiver routine:
 Master:SPI1_SCK(PA5)\SPI1_MISO(PA6)\SPI1_MOSI(PA7).
 Slave:SPI1_SCK(PA5)\SPI1_MISO(PA6)\SPI1_MOSI(PA7).

 This example demonstrates that Master and Slave use DAM full-duplex transmission
 and reception at the same time.
 Note: The two boards download the Master and Slave programs respectively, and power
 on at the same time.
 Hardware connection:PA5 -- PA5
                     PA6 -- PA6
                     PA7 -- PA7

*/
#include "debug.h"
#include "string.h"

/* Global define */
#define Size          18

/* Global Variable */
u8 TxData1[Size] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
                    0x11, 0x12, 0x13, 0x14, 0x15, 0x16,
                    0x21, 0x22, 0x23, 0x24, 0x25, 0x26};
u8 TxData2[Size] = {0x31, 0x32, 0x33, 0x34, 0x35, 0x36,
                    0x41, 0x42, 0x43, 0x44, 0x45, 0x46,
                    0x51, 0x52, 0x53, 0x54, 0x55, 0x56};

u8 RxData1[Size] = {0};
u8 RxData2[Size] = {0};

/*********************************************************************
 * @fn      SPI_FullDuplex_Init
 *
 * @brief   Configuring the SPI for full-duplex communication.
 *
 * @return  none
 */
void SPI_FullDuplex_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};
    SPI_InitTypeDef  SPI_InitStructure  = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_SPI1, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);

    // SPI1 HOST
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;                       // Ƭѡ
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_SetBits(GPIOA, GPIO_Pin_4);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;                       // ʱ
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;                       // (MOSI)
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    // SPI2 SLAVE
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;                      // Ƭѡ
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;                   // SPIƬѡźͷʱӦָߵƽ
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;                      // SPI2ʱ
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14;                      // SPI2(MISO)
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    // SPI1 HOST
    SPI_InitStructure.SPI_Direction = SPI_Direction_1Line_Tx;       // ߰˫״̬
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;                   // 
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;                       // Ƭѡ
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_LSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_Init(SPI1, &SPI_InitStructure);

    // SPI2 SLAVE
    SPI_InitStructure.SPI_Direction = SPI_Direction_1Line_Rx;       // ߰˫״̬
    SPI_InitStructure.SPI_Mode = SPI_Mode_Slave;                    // ӻ
    SPI_InitStructure.SPI_NSS = SPI_NSS_Hard;                       // ӲƬѡ
    SPI_Init(SPI2, &SPI_InitStructure);

    SPI_Cmd(SPI1, ENABLE);
    SPI_Cmd(SPI2, ENABLE);
}

/*********************************************************************
 * @fn      SPI_DMA_Init
 *
 * @brief   Initializes the DMAy Channelx configuration.
 *
 * @param   DMA_CHx - x can be 1 to 7.
 *          ppadr - Peripheral base address.
 *          memadr - Memory base address.
 *          bufsize - DMA channel buffer size.
 *
 * @return  none
 */
void SPI_DMA_Init(void)
{
    DMA_InitTypeDef DMA_InitStructure = {0};

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

    // SPI1 HOST Tx
    DMA_DeInit(DMA1_Channel3);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32)&SPI1->DATAR;
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32)TxData1;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
    DMA_InitStructure.DMA_BufferSize = Size;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
    DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
    DMA_Init(DMA1_Channel3, &DMA_InitStructure);
    SPI_I2S_DMACmd(SPI1, SPI_I2S_DMAReq_Tx, ENABLE);

    // SPI2 SLAVE Tx
    DMA_DeInit(DMA1_Channel5);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32)&SPI2->DATAR;
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32)TxData2;
    DMA_Init(DMA1_Channel5, &DMA_InitStructure);
    SPI_I2S_DMACmd(SPI2, SPI_I2S_DMAReq_Tx, ENABLE);

    // SPI1 HOST Rx
    DMA_DeInit(DMA1_Channel2);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32)&SPI1->DATAR;
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32)RxData1;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
    DMA_InitStructure.DMA_BufferSize = Size;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
    DMA_InitStructure.DMA_Priority = DMA_Priority_High;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
    DMA_Init(DMA1_Channel2, &DMA_InitStructure);
    SPI_I2S_DMACmd(SPI1, SPI_I2S_DMAReq_Rx, ENABLE);
    DMA_Cmd(DMA1_Channel2, ENABLE);

    // SPI2 SLAVE Rx
    DMA_DeInit(DMA1_Channel4);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32)&SPI2->DATAR;
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32)RxData2;
    DMA_Init(DMA1_Channel4, &DMA_InitStructure);
    SPI_I2S_DMACmd(SPI2, SPI_I2S_DMAReq_Rx, ENABLE);
    DMA_Cmd(DMA1_Channel4, ENABLE);
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    u8 i = 0, j = 0;

    Delay_Init();
    USART_Printf_Init(115200);
    printf("SystemClk:%d\r\n", SystemCoreClock);

    SPI_FullDuplex_Init();
    SPI_DMA_Init();

    printf("SPI1 Tx...\r\n");
    GPIO_ResetBits(GPIOA, GPIO_Pin_4);
    DMA_Cmd(DMA1_Channel3, ENABLE);                         // SPI1ݷ
    while(DMA_GetFlagStatus(DMA1_FLAG_TC3) == 0);           // ȴDMASPI1
    while(DMA_GetFlagStatus(DMA1_FLAG_TC4) == 0);           // ȴӻDMASPI2
    GPIO_SetBits(GPIOA, GPIO_Pin_4);                        // ͷƬѡź

    for(i = 0; i < Size; i++)                               // ӡSPI2յ
    {
        printf("SPI2_RxData[%d]:%x\r\n", i, RxData2[i]);
    }

    printf("SPI2 Tx...\r\n");
    SPI2->CTLR1 |= 1<<14;                                   // ӻSPI2лΪ״̬
    /* ׼õһҪ͵ݣȴƬѡʱź
     * SPIʱӽϸʱݼĴδɸµ
     * */
    DMA_Cmd(DMA1_Channel5, ENABLE);                         // ӻSPI2ݷ
    GPIO_ResetBits(GPIOA, GPIO_Pin_4);                      // Ƭѡ
    SPI1->CTLR1 &= ~(1<<14);                                // SPI1лΪ״̬,״̬лɺSPI1ʱź

    while(DMA_GetFlagStatus(DMA1_FLAG_TC5) == 0);           // ȴӻDMASPI2
    // ȴSPI2
    __NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();
    __NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();
    __NOP();__NOP();__NOP();__NOP();
    GPIO_SetBits(GPIOA, GPIO_Pin_4);
    while(DMA_GetFlagStatus(DMA1_FLAG_TC2) == 0);           // ȴDMASPI1
    SPI1->CTLR1 |= 1<<14;                                   // SPI1лΪ״̬
    SPI2->CTLR1 &= ~(1<<14);                                // ӻSPI2лΪ״̬

    for(j = 0; j < Size; j++)                               // ӡSPI1յ
    {
        printf("SPI1_RxData[%d]:%x\r\n", j, RxData1[j]);
    }

    while(1);

}


